@@ -89,6 +89,13 @@ public:
    */
   void AddInstance(nsIInstanceElementPrivate *aInstance);
 
+  /**
+   * Remove an instance element
+   *
+   * @param aInstance         The instance element
+   */
+  void RemoveInstance(nsIInstanceElementPrivate *aInstance);
+
   /**
    * Get the instance document at a given index
    *
@@ -109,6 +116,101 @@ protected:
   nsCOMArray<nsIInstanceElementPrivate>    mInstanceList;
 };
 
+/**
+ * A class for storing pointers to XForms controls added to an XForms
+ * model. Organized as a tree, with pointers to first child and next sibling.
+ *
+ * Notes:
+ * 1) The root node is special; only has children and a nsnull mNode.
+ * 2) All functions operate on the node they are called on, and its
+ * subtree. Functions will never go up (as in: nearer the root) in the tree.
+ */
+class nsXFormsControlListItem
+{
+  /** The XForms control itself */
+  nsCOMPtr<nsIXFormsControl>      mNode;
+
+  /** The next sibling of the node */
+  nsXFormsControlListItem        *mNextSibling;
+
+  /** The first child of the node */
+  nsXFormsControlListItem        *mFirstChild;
+
+public:
+  nsXFormsControlListItem(nsIXFormsControl* aControl);
+  ~nsXFormsControlListItem();
+  nsXFormsControlListItem(const nsXFormsControlListItem& aCopy);
+
+  /** Clear contents of current node, all siblings, and all children */
+  void Clear();
+
+  /**
+   * Remove a control from the current (sub-) tree. Will search from that node
+   * and down in the tree for the node.
+   *
+   * @param aControl     The control to remove
+   * @param aRemoved     Was the control found and removed?
+   */
+  nsresult RemoveControl(nsIXFormsControl *aControl, PRBool &aRemoved);
+
+  /**
+   * Add a control to the (sub-) tree as a child to the given |aParent|. If
+   * there is no |aParent|, it will insert it as a sibling to |this|. If
+   * |aParent| is not found, it will bail.
+   *
+   * @param aControl     The control to insert
+   * @param aParent      The (eventual) parent to insert it under
+   */
+  nsresult AddControl(nsIXFormsControl *aControl,
+                      nsIXFormsControl *aParent);
+
+  /**
+   * Find a control in the (sub-) tree.
+   *
+   * @param aControl     The control to find
+   */
+  nsXFormsControlListItem* FindControl(nsIXFormsControl *aControl);
+
+  /**
+   * Return the nsIXFormsControl that this node contains.
+   */
+  already_AddRefed<nsIXFormsControl> Control();
+
+  /** Return the first child of the node */
+  nsXFormsControlListItem* FirstChild() { return mFirstChild; };
+
+  /** Return the next sibling of the node */
+  nsXFormsControlListItem* NextSibling() { return mNextSibling; };
+
+  /**
+   * An iterator implementation for the class.
+   */
+  class iterator
+  {
+  private:
+    /** The control the iterator is currently pointing at */
+    nsXFormsControlListItem  *mCur;
+
+    /** A stack of non-visited nodes */
+    nsVoidArray               mStack;
+
+  public:
+    iterator();
+    iterator(const iterator&);
+    iterator operator=(nsXFormsControlListItem*);
+    bool operator!=(const nsXFormsControlListItem*);
+    iterator operator++();
+    nsXFormsControlListItem* operator*();
+  };
+  friend class nsXFormsControlListItem::iterator;
+
+  /** The begining position for the node (itself) */
+  nsXFormsControlListItem* begin();
+
+  /** The end position for the node (nsnull) */
+  nsXFormsControlListItem* end();
+};
+
 /**
  * Implementation of the XForms \<model\> element.
  *
@@ -146,7 +248,17 @@ public:
   NS_IMETHOD OnCreated(nsIXTFGenericElementWrapper *aWrapper);
 
   // nsIXFormsControlBase overrides
-  NS_IMETHOD Bind() {
+  NS_IMETHOD Bind(PRBool *aContextChanged) {
+    // dummy method, so does nothing
+    return NS_OK;
+  };
+
+  NS_IMETHOD GetOnDeferredBindList(PRBool *onList) {
+    // dummy method, so does nothing
+    return NS_OK;
+  };
+
+  NS_IMETHOD SetOnDeferredBindList(PRBool putOnList) {
     // dummy method, so does nothing
     return NS_OK;
   };
@@ -163,16 +275,31 @@ public:
    * @param aControl          XForms control waiting to be bound
    */
   static NS_HIDDEN_(nsresult) DeferElementBind(nsIDOMDocument    *aDoc,
-                                               nsIXFormsControlBase  *aControl);
+                                               nsIXFormsControl  *aControl);
 
   static nsresult NeedsPostRefresh(nsIXFormsControl* aControl);
+
+  // Sometimes a child that is on the post refresh list will want to force
+  // its containing xforms control to refresh.  Like if an xf:item's label
+  // refreshes, it wants to reflect any of its changes in the xf:select or
+  // xf:select1 that contains it.  This function will try to minimize that by
+  // allowing xforms controls that function as containers (like select/select1)
+  // to defer their refresh until most or all of its children have refreshed.
+  // Well, at least until every control on the sPostRefreshList has been
+  // processed.  This will return PR_TRUE if the refresh will be deferred.
+  static PRBool ContainerNeedsPostRefresh(nsIXFormsControl* aControl);
   static void CancelPostRefresh(nsIXFormsControl* aControl);
+
+  /**
+   * Returns the DOM element for the model.
+   */
+  already_AddRefed<nsIDOMElement> GetDOMElement();
+
 private:
 
   NS_HIDDEN_(already_AddRefed<nsIDOMDocument>)
     FindInstanceDocument(const nsAString &aID);
   NS_HIDDEN_(void)     Reset();
-  NS_HIDDEN_(void)     Ready();
   NS_HIDDEN_(void)     BackupOrRestoreInstanceData(PRBool restore);
 
   /** Initializes the MIPs on all form controls */
@@ -189,24 +316,11 @@ private:
                                    nsIDOMNode                *aContextNode,
                                    PRInt32                    aContextPosition,
                                    PRInt32                    aContextSize,
-                                   nsIDOMElement             *aBindElement);
+                                   nsIDOMElement             *aBindElement,
+                                   PRBool                     aIsOuter = PR_FALSE);
 
   NS_HIDDEN_(void)     RemoveModelFromDocument();
 
-  /**
-   * Set the states on the control |aControl| bound to the instance data node
-   * |aNode|. It dispatches the necessary events and sets the pseudo class
-   * states. |aAllStates| determines whether all states should be set, or only
-   * changed.
-   *
-   * @param aControl          The event target
-   * @param aNode             The instance node
-   * @param aAllStates        Set all states (PR_TRUE), or only changed
-   */
-  NS_HIDDEN_(nsresult) SetStatesInternal(nsIXFormsControl *aControl,
-                                         nsIDOMNode       *aNode,
-                                         PRBool            aAllStates = PR_FALSE);
-
   /**
    * Sets the state of a specific state.
    *
@@ -219,8 +333,11 @@ private:
                                       nsXFormsEvent  aOnEvent);
 
   /**
-   * Call the Bind() and Refresh() on controls which was deferred because
-   * the model was not ready.
+   * Handle controls bindings which was deferred because the model was not
+   * ready.
+   *
+   * @note Only registers the controls with the model. Does not setup
+   * bindings, etc.
    *
    * @param aDoc              Document that contains the XForms control
    */
@@ -240,10 +357,40 @@ private:
    */
   NS_HIDDEN_(nsresult) HandleUnload(nsIDOMEvent *aEvent);
 
+  NS_HIDDEN_(nsresult) RefreshSubTree(nsXFormsControlListItem *aCurrent,
+                                      PRBool                   aForceRebind);
+
+  NS_HIDDEN_(nsresult) ValidateDocument(nsIDOMDocument *aInstanceDocument,
+                                        PRBool         *aResult);
+
+  /**
+   * Request to send an update event to the model. If an update is already
+   * running, the event will be queued, and sent after that. If multiple
+   * events are queued, they will be dispatched FIFO order.
+   *
+   * @param aEvent            The requested event
+   */
+  NS_HIDDEN_(nsresult) RequestUpdateEvent(nsXFormsEvent aEvent);
+  
+  /**
+   * Returns true if a schema has already been registered to address the same
+   * namespace as aSchemaElement.  Per section 3.3.1 of the XForms spec:
+   * Within each XForms Model, there is a limit of one Schema per namespace
+   * declaration, including inline and linked Schemas.
+   *
+   * @param aSchemaElement The schema element
+   */
+  PRBool IsDuplicateSchema(nsIDOMElement *aSchemaElement);
+
+  /**
+   * Validate all the instance documents.
+   */
+  void ValidateInstanceDocuments();
+
   nsIDOMElement            *mElement;
   nsCOMPtr<nsISchemaLoader> mSchemas;
   nsStringArray             mPendingInlineSchemas;
-  nsVoidArray               mFormControls;
+  nsXFormsControlListItem   mFormControls;
 
   PRInt32 mSchemaCount;
   PRInt32 mSchemaTotal;
@@ -258,15 +405,58 @@ private:
    */
   nsCOMArray<nsIDOMNode>    mChangedNodes;
 
-  // This flag indicates whether or not the document fired DOMContentLoaded
-  PRBool mDocumentLoaded;
+  /**
+   * This flag indicates whether or not the document has fired
+   * DOMContentLoaded
+   */
+  PRPackedBool mDocumentLoaded;
 
-  // This flag indicates whether a xforms-rebuild has been called, but no
-  // xforms-revalidate yet
-  PRBool mNeedsRefresh;
+  /**
+   * Indicates whether all controls should be refreshed on the next Refresh()
+   * run.
+   */
+  PRPackedBool mRebindAllControls;
 
-  // This flag indicates whether instance elements have been initialized
-  PRBool mInstancesInitialized;
+  /**
+   * Indicates whether instance elements have been initialized
+   */
+  PRPackedBool mInstancesInitialized;
+
+  /**
+   * Indicates whether the model has handled the xforms-ready event
+   */
+  PRPackedBool mReadyHandled;
+
+  /**
+   * Indicates whether the model's instance was built by lazy authoring
+   */
+  PRPackedBool mLazyModel;
+
+  /**
+   * Indicates whether the model has handled the xforms-model-construct-done
+   * event
+   */
+  PRPackedBool mConstructDoneHandled;
+
+  /**
+   * Indicates whether the model is currently processing an update event,
+   * ie. xforms-rebuild, xforms-recalculate, xforms-revalidate, or
+   * xforms-refresh.
+   */
+  PRPackedBool mProcessingUpdateEvent;
+
+  /**
+   * A list of update events that have been queued, because they were
+   * requested while another update was running.
+   */
+  nsVoidArray mUpdateEventQueue;
+
+  /**
+   * The maximum allowed number of iterations of queued event dispatching in
+   * RequestUpdateEvent(), before there is believe to be a loop, and
+   * processing stops.
+   */
+  PRInt32 mLoopMax;
 
   /**
    * All instance documents contained by this model, including lazy-authored
@@ -274,9 +464,6 @@ private:
    */
   nsRefPtr<nsXFormsModelInstanceDocuments> mInstanceDocuments;
 
-  /** Indicates whether the model's instance was built by lazy authoring */
-  PRBool mLazyModel;
-
   /**
    * Type information for nodes, with their type set through \<xforms:bind\>.
    *
@@ -291,6 +478,8 @@ private:
    * @see http://www.w3.org/TR/xforms/slice6.html#model-prop-p3ptype
    */
   nsClassHashtable<nsISupportsHashKey, nsString> mNodeToP3PType;
+
+  friend class Updating;
 };
 
 /**
