@@ -1,3 +1,39 @@
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
+#
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
+#
+# The Original Code is The Browser Profile Migrator.
+#
+# The Initial Developer of the Original Code is Ben Goodger.
+# Portions created by the Initial Developer are Copyright (C) 2004
+# the Initial Developer. All Rights Reserved.
+#
+# Contributor(s):
+#   Ben Goodger <ben@bengoodger.com>
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
+
 const kIMig = Components.interfaces.nsIMailProfileMigrator;
 const kIPStartup = Components.interfaces.nsIProfileStartup;
 const kProfileMigratorContractIDPrefix = "@mozilla.org/profile/migrator;1?app=mail&type=";
@@ -8,14 +44,10 @@ var MigrationWizard = {
   _selectedProfile: null,       // Selected Profile name to import from
   _wiz: null,
   _migrator: null,
-  _windowInitialized: false,
   _autoMigrate: null,
 
   init: function ()
   {
-    if (this._windowInitialized)
-      return;
-	  
     var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
     os.addObserver(this, "Migration:Started", false);
     os.addObserver(this, "Migration:ItemBeforeMigrate", false);
@@ -27,15 +59,32 @@ var MigrationWizard = {
 
     if ("arguments" in window) {
       this._source = window.arguments[0];
-      this._autoMigrate = window.arguments[1].QueryInterface(kIPStartup);
+      this._migrator = window.arguments[1] ? window.arguments[1].QueryInterface(kIMig) : null;
+      this._autoMigrate = window.arguments[2].QueryInterface(kIPStartup);
       
       // Show the "nothing" option in the automigrate case to provide an
       // easily identifiable way to avoid migration and create a new profile.
       var nothing = document.getElementById("nothing");
       nothing.hidden = false;      
     }
+
+    this.onImportSourcePageShow();
 	
-    this._windowInitialized = true;
+    // Behavior alert! If we were given a migrator already, then we are going to perform migration
+    // with that migrator, skip the wizard screen where we show all of the migration sources and 
+    // jump right into migration.
+    if (this._migrator)
+    {
+      if (this._migrator.sourceHasMultipleProfiles)
+        this._wiz.goTo("selectProfile");
+      else 
+      {
+        var sourceProfiles = this._migrator.sourceProfiles;
+        var profileName = sourceProfiles.QueryElementAt(0, Components.interfaces.nsISupportsString);
+        this._selectedProfile = profileName.data;
+        this._wiz.goTo("migrating");
+      }
+    }
   },
   
   uninit: function ()
@@ -51,10 +100,7 @@ var MigrationWizard = {
   // 1 - Import Source
   onImportSourcePageShow: function ()
   {
-    if (!this._windowInitialized)
-      this.init();
-	  
-    document.documentElement.getButton("back").disabled = true;
+    this._wiz.canRewind = false;
     
     // Figure out what source apps are are available to import from:
     var group = document.getElementById("importSourceGroup");
@@ -97,14 +143,14 @@ var MigrationWizard = {
       this._itemsFlags = kIMig.ALL;
       this._selectedProfile = null;
     }
-    if (!this._autoMigrate)
+    
       this._source = newSource;
       
     // check for more than one source profile
     if (this._migrator.sourceHasMultipleProfiles)
       this._wiz.currentPage.next = "selectProfile";
     else {
-      this._wiz.currentPage.next = this._autoMigrate ? "migrating" : "importItems";
+      this._wiz.currentPage.next = "migrating";
       var sourceProfiles = this._migrator.sourceProfiles;
       if (sourceProfiles && sourceProfiles.Count() == 1) {
         var profileName = sourceProfiles.QueryElementAt(0, Components.interfaces.nsISupportsString);
@@ -203,15 +249,15 @@ var MigrationWizard = {
       }
     }
 
-    this._wiz.getButton("next").disabled = !oneChecked;
+    this._wiz.canAdvance = oneChecked;
   },
   
   // 4 - Migrating
   onMigratingPageShow: function ()
   {
     this._wiz.getButton("cancel").disabled = true;
-    this._wiz.getButton("back").disabled = true;
-    this._wiz.getButton("next").disabled = true;
+    this._wiz.canRewind = false;
+    this._wiz.canAdvance = false;
     
     // When automigrating, show all of the data that can be received from this source.
     if (this._autoMigrate)
@@ -274,12 +320,13 @@ var MigrationWizard = {
       dump("*** done\n");
       if (this._autoMigrate) {
         // We're done now.
+        this._wiz.canAdvance = true;
         this._wiz.advance();
         setTimeout("close()", 5000);
       }
       else {
+        this._wiz.canAdvance = true;
         var nextButton = this._wiz.getButton("next");
-        nextButton.disabled = false;
         nextButton.click();
       }
       break;
@@ -292,45 +339,7 @@ var MigrationWizard = {
   onDonePageShow: function ()
   {
     this._wiz.getButton("cancel").disabled = true;
-    this._wiz.getButton("back").disabled = true;
-    this._wiz.getButton("finish").disabled = false;
+    this._wiz.canRewind = false;
     this._listItems("doneItems");
   }
 };
-
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is The Browser Profile Migrator.
-#
-# The Initial Developer of the Original Code is Ben Goodger.
-# Portions created by the Initial Developer are Copyright (C) 2004
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Ben Goodger <ben@bengoodger.com>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
-
